require 'date'

module BcaParser
  module TimeKeeper

    DAYS = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
    MONTHS = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
    MAX_DAYS = 2678400

    def self.twoPreviousMonths
      month = Time.new.month
      [month - 2, month - 1]
    end

    def self.checkMonthForBalance(month)
      month = month.to_i
      now_month = Time.now.month

      if month >= now_month or month < now_month - 2
        false
      else
        true
      end
    end

    def self.checkDateForBalance(start_date, start_month, end_date, end_month)
      start_day = TimeKeeper.makeDateFromString(start_date, start_month, Time.now.year)
      end_day = TimeKeeper.makeDateFromString(end_date, end_month, Time.now.year)
      if !compareDate(start_day, end_day) 
        return false
      end

      if !nowAndPast?(start_day, end_day)
        return false
      end
  
      if !inRangeMaxDays?(start_day, end_day)
        return false
      else
        return true
      end
    end

    def self.nowAndPast?(start_day, end_day)
      now = Time.now
      if start_day <= now && end_day <= now
        return true
      else
        return false
      end
    end

    def self.inRangeMaxDays?(start_day, end_day)
      if end_day - start_day <= MAX_DAYS
        return true
      else
        return false
      end
    end

    def self.checkLeapYear(year)
      year % 400 == 0 || (year % 4 == 0 && year % 100 != 0) ? true : false
    end

    def self.makeDateFromString(day, month, year)
      date = Time.new(year.to_i, month.to_i, day.to_i)
    end

    def self.compareDate(date1, date2)
      date2 >= date1 ? true : false
    end

    def self.getIndonesianMonth(month)
      month = month.to_i
      case month
      when 1
        'Januari'
      when 2
        'Februari'
      when 3
        'Maret'
      when 4
        'April'
      when 5
        'Mei'
      when 6
        'Juni'
      when 7
        'Juli'
      when 8
        'Agustus'
      when 9
        'September'
      when 10
        'Oktober'
      when 11
        'Nopember'
      when 12
        'Desember'
      end
    end

    def self.checkDayMonth(day, month, year=nil)
      day = day.to_i
      month = month.to_i

      if !MONTHS.include?(month)
        return false
      end
      year ||= Time.now.year
      if checkLeapYear(year)
        if month == 2
          total_days = 29
        end
      else
        total_days = DAYS[month-1]
      end

      if day > 0 and day <= total_days
        return true
      else
        return false
      end
      
    end
  end
end